/*
 * Copyright (C) 07 bplan GmbH. All rights reserved.
 *
 * CPU header for ARM
 *
*/

#ifndef __CPU_ARM_H__
#define __CPU_ARM_H__

/**********/
/**********/

// PSR
#define PSR_M0			(1<<0)
#define PSR_M1			(1<<1)
#define PSR_M2			(1<<2)
#define PSR_M3			(1<<3)
#define PSR_M4			(1<<4)
#define PSR_M_MASK	(0x1f)

#define PSR_T 			(1<<5)
#define PSR_F 			(1<<6)
#define PSR_I 			(1<<7)

#define PSR_Q 			(1<<27)
#define PSR_V 			(1<<28)
#define PSR_C 			(1<<29)
#define PSR_Z 			(1<<30)
#define PSR_N 			(1<<31)

#define PSR_MX			(0x1f)

#define PSR_M_USER	      (0x10)
#define PSR_M_FIQ 	      (0x11)
#define PSR_M_IRQ 	      (0x12)
#define PSR_M_SUPERVISOR	(0x13)
#define PSR_M_ABORT     	(0x17)
#define PSR_M_UNDEFINED  	(0x1B)
#define PSR_M_SYSTEM    	(0x1F)

/**********/
/**********/

// Control register 1
#define CTRLREG_M			(1<<0)
#define CTRLREG_A			(1<<1)
#define CTRLREG_C			(1<<2)
#define CTRLREG_W			(1<<3)

#define CTRLREG_P			(1<<4)
#define CTRLREG_D			(1<<5)
#define CTRLREG_L			(1<<6)
#define CTRLREG_B			(1<<7)

#define CTRLREG_S			(1<<8)
#define CTRLREG_R			(1<<9)
#define CTRLREG_F			(1<<10)
#define CTRLREG_Z			(1<<11)

#define CTRLREG_I			(1<<12)
#define CTRLREG_V			(1<<13)
#define CTRLREG_RR		(1<<14)
#define CTRLREG_L4		(1<<15)

/**********/
/**********/

// Cache type reg
#define CACHETYPE_REG_LINE_8BYTES		 	(0x0)
#define CACHETYPE_REG_LINE_16BYTES			(0x1)
#define CACHETYPE_REG_LINE_32BYTES			(0x2)
#define CACHETYPE_REG_LINE_64BYTES			(0x3)

/**********/

// define in OFW ARM binding
#define MMU_FIRMWARE_VIRTBASE		0xf7000000

// it makes 16MB for the FW which HAS to be enough
#define MMU_FIRMWARE_IOASE			0xf8000000

// MMU

#define  MMU_TINYPAGE_SIZE			(1*1024)
#define  MMU_SMALLPAGE_SIZE			(4*1024)
#define  MMU_LARGEPAGE_SIZE			(64*1024)


#define  MMU_SECT_SIZE			(1*1024*1024)

#define  MMU_SECT_TABLEMASK  			0xfff00000
#define  MMU_SECT_SECTMASK  			~MMU_SECT_TABLEMASK

/**/
#define MMU_1STLEVEL_UNDEX			0x0
#define MMU_1STLEVEL_COARSE			0x1
#define MMU_1STLEVEL_SECTION		0x2
#define MMU_1STLEVEL_FINE   		0x3
#define MMU_1STLEVEL_GET(__x)  ( (__x) & 0x3)

#define MMU_SECT_BASE_SET(__x)			( ( ( (uint32_t) __x) & 0xfff00000)  )
#define MMU_SECT_BASE_GET(__x)			( ( ( (uint32_t)__x) & 0xfff00000)  )

#define MMU_FIRSTLV_DOMAIN_SET(__x) 	(	( (__x) & 0xf)   <<  5 )
#define MMU_FIRSTLV_DOMAIN_GET(__x) 	(	( (__x) >> 5 )   & 0xf )

#define MMU_SECT_AP_SET(__x) 			( ( (__x) & 0x3)   << 10 )
#define MMU_SECT_AP_GET(__x) 		( ( (__x) >> 10)   & 0x3 )

#define MMU_SECT_CB_MASK 		( 0xc )

#define MMU_SECT_BUFFERABLE 	(1<<2)
#define MMU_SECT_CACHEABLE 		(1<<3)

/**/
#define MMU_PAGE_TYPE_LARGE 	0x1
#define MMU_PAGE_TYPE_SMALL	  0x2
#define MMU_PAGE_TYPE_TINY 	  0x3

#define MMU_PAGE_AP_OFFSET			4
#define MMU_TINYPAGE_APMASK  	  (0x30) 
#define MMU_SMALLPAGE_APMASK    (0xFF0) 
#define MMU_LARGEPAGE_APMASK    (0xFF0) 

#define MMU_LARGEPAGE_ADDRMASK (0xFFFF0000) 
#define MMU_SMALLPAGE_ADDRMASK (0xFFFFF000) 
#define MMU_TINYPAGE_ADDRMASK  (0xFFFFFC00) 

#define MMU_FIRSTLV_COARSEPAGE_MASK 		0xFFFFFC00
#define MMU_FIRSTLV_FINEPAGE_MASK 		  0xFFFFF000
#define MMU_FIRSTLV_COARSEPAGE_BASE_SET(__x)		( ( (uint32_t) (__x) & 0xFFFFFC00)  )
#define MMU_FIRSTLV_COARSEPAGE_BASE_GET(__x)		( ( (uint32_t) (__x) & 0xFFFFFC00)  )

#define MMU_FIRSTLV_FINEPAGE_BASE_SET(__x)			( ( (uint32_t) (__x) & 0xFFFFF000)  )
#define MMU_FIRSTLV_FINEPAGE_BASE_GET(__x)			( ( (uint32_t) (__x) & 0xFFFFF000)  )
/**/

#define MMU_DOMAIN_CLIENT 		(0x1)
#define MMU_DOMAIN_MANAGER 		(0x3)

#define MMU_DOMAIN_OFW				0
#define MMU_DOMAIN_MEM				1
#define MMU_DOMAIN_IO					2

#define MMU_DOMAIN_CLIENT 		(0x1)
#define MMU_DOMAIN_MANAGER 		(0x3)


#define MAKE_DOMAIN(__domain,__type) ( (__type) << ( (__domain)*2) )

#define MMU_COMPUTE_ENTRY(__x)  ( ( ( (__x) & MMU_SECT_TABLEMASK) )  >> 18) / 4

#endif /* __CPU_ARM_H__ */
