# TMPA910/Topas memory config


# Burst = 2, // not really found
# 32bit data, 9 colums, 13 rows
/*
. dmc_memory_cfg_3 (DMC Memory Configuration Register)
               Address = (0xF430_0000) + (0x000C)

[31:23]                     Undefined Read undefined. Write as zero.
        -               -
[22:21] active_chips    R/W 0y00      Number of memory chips that can
                                      generate the Refresh command:
                                      0y00 = 1 chip
                                      0y01 = Reserved
                                      0y10 = Reserved
                                      0y11 = Reserved
[20:18]                     Undefined Read undefined. Write as zero.
        -               -
[17:15] memory_burst    R/W 0y010     Set the read and write access burst
                                      length for the SDRAM
                                      0y000 = Burst 1
                                      0y001 = Burst 2
                                      0y010 = Burst 4
                                      0y011 = Burst 8
                                      0y100 = Burst 16
                                      other = Reserved
[14]    stop_mem_clock  R/W 0y0       memory clock stop:
                                      0y1 = Enable
                                      0y0 = Disable
[13]    auto_power_down R/W 0y0       SDRAM auto Power down Enable
                                      0y1 = Enable
                                      0y0 = Disable
[12:7]  power_down_prd  R/W 0y000000  Number       of    SDRAM      automatic
                                      power-down memory clocks:
                                      (Min. value = 1)
                                      0y000001  0y111111
[6]     ap_bit          R/W 0y0       the position of the auto-precharge bit in
                                      the memory address:
                                      0y0 = address bit 10
                                      0y1 = address bit 8.
[5:3]   row_bits        R/W 0y100     The number of row address bits:
                                      0y000 = 11 bits
                                      0y001 = 12 bits
                                      0y010 = 13 bits
                                      0y011 = 14 bits
                                      0y100 = 15 bits
                                      0y101 = 16 bits.
                                      other = Reserved
[2:0]   column_bits     R/W 0y000     The number of column address bits:
                                      0y000 = 8 bits
                                      0y001 = 9 bits
                                      0y010 = 10 bits
                                      0y011 = 11 bits
                                      0y100 = 12 bits.
                                      other = Reserved
*/


#if 1
// calculated for 167 Mhz 
// *** MEMORY WRITE PERFORMANCE (256 MB LOOP) ***
// size =   8388608 bytes: 125.490 MB/s
// *** MEMORY READ PERFORMANCE (256 MB LOOP) ***
// size =   8388608 bytes: 112.281 MB/s


_micron_mt48CL16M_6a_config:
	.word 	0x06 		// 0x14, CAS = 3
	.word 	0x00 		// 0x18, DQSS must be 0, see TMPA910 doc
	.word 	0x02 		// 0x1c, MRD = 2
	.word 	0x07 		// 0x20, RAS = 7
	.word 	0x0a 		// 0x24, RC  = 10
	.word 	0x13 		// 0x28, RCD = 3
	.word 	0x10a		// 0x2c, RFC = 10
	.word 	0x13 		// 0x30, RP  = 3
	.word 	0x02 		// 0x34, RRD = 2
	.word 	0x02 		// 0x38, WR  = 2
	.word 	0x01 		// 0x3c, WTR = 1
	.word 	0x0a 		// 0x40, XP  = 10 - undocumented 
	.word 	0x0b 		// 0x44, XSR = 11
	.word 	0x14 		// 0x48, ESR = 20 - undocumented 

_micron_mt48CL16M_6a_autorefresh:
	.word (0x0a60)
	//.word (0x2000)

_micron_mt48CL16M_6a_memconfreg:
	.word (0x1 | (0x2 << 3) | (0x1 << 15) )

#else
// calculated for 96 Mhz 
_micron_mt48CL16M_6a_config:
	.word 	0x04 		// 0x14, CAS = 2
	.word 	0x00 		// 0x18, DQSS must be 0, see TMPA910 doc
	.word 	0x02 		// 0x1c, MRD = 2
	.word 	0x06 		// 0x20, RAS = 4
	.word 	0x08 		// 0x24, RC  = 6
	.word 	0x16 		// 0x28, RCD = 2
	.word 	0x108		// 0x2c, RFC = 6
	.word 	0x12 		// 0x30, RP  = 2
	.word 	0x02 		// 0x34, RRD = 2
	.word 	0x02 		// 0x38, WR  = 2
	.word 	0x01 		// 0x3c, WTR = 1
	.word 	0x0a 		// 0x40, XP  = 10 - undocumented 
	.word 	0x08 		// 0x44, XSR = 6
	.word 	0x14 		// 0x48, ESR = 20 - undocumented 

_micron_mt48CL16M_6a_autorefresh:
	.word (0x0a60)

_micron_mt48CL16M_6a_memconfreg:
	.word (0x1 | (0x2 << 3) | (0x2 << 15) )

#endif
