/*
 * Copyright (C) 2008 bplan GmbH. All rights reserved.
 *
 * Boot strap for TMPA910
 * ARM/asm code
 *
 * - clocks setup
*/

.include "low_defs.i"

/********/
/********/

	.align

	.section ".text"
	.type	_init_clocks,#function
	.globl	_init_clocks


.set SYSCR0, 0x000
.set SYSCR1, 0x004
.set SYSCR2, 0x008
.set		SYSCR2_LUPFLAG,  	0x01
.set  	SYSCR2_FCSEL_PLL, 0x02

.set SYSCR3, 0x00C
.set SYSCR4, 0x010
.set SYSCR5, 0x014
.set SYSCR6, 0x018
.set SYSCR7, 0x01C

.set CLKCR5, 0x054

/********/
/********/
_init_clocks:
	get_clockbase r0

	@ make sure the gear is 0 -> 1:1
	mov r1,#0
	mov r1,#0
	str r1,[r0,#SYSCR1]

	@ switch to PLL * 8
/*
 * [7]   PLLON R/W 0y0       PLL operation control
 *                             0: OFF
 *                             1: ON
 *       -     -
 * [6]             Undefined Read undefined. Write as zero.
 * [5]   C2S   R/W 0y1       PLL constant value setting1
 *                               Always write as zero
 * [4:0] ND    R/W 0y00111   PLL constant value setting - 2
 *                             b00101 for x6, b00111 for x8
*/	
  // Value for *8
	mov r1,# (0x7 | (1<<5) | (1<<7) )

	// Value for *6
	//mov r1,# (0x5 | (1<<5) | (1<<7) )
	str r1,[r0,#SYSCR3]

_init_clocks_pool:
	@ pool until done. beq -> if 0 branch
	@ [0] LUPFLAG RO 0y0 End flag for the lock-up counter for the PLL
	@                      0 : Not end, before PLL execution
	@                      1 : End

	ldr r1,[r0,#SYSCR2]
	ands	r1, r1, #SYSCR2_LUPFLAG
	beq	_init_clocks_pool

	@ PLL ready, select it
	mov r1,#SYSCR2_FCSEL_PLL
	str r1,[r0,#SYSCR2]	


	# setup the syscr4. The documentation to not specifiy if
	# it should be done before or after
	# the PLL switch
/*  SYSCR4 (System Control Register 4)
 * [7:4] RS R/W 0y0111 PLL constant value setting - 3
 *                                  x8                            x6
 *                       Equal and        Less than    Equal and     Less than
 *                      Over 140MHz        140MHz     Over 140MHz     140MHz
 *                        0y0110           0y1001       0y0110        0y0111
 * [3:2] IS R/W 0y10   PLL constant value setting - 4
 *                     Always write "01"
 * [1:0] FS R/W 0y01   PLL constant value setting  5
 *                                  x8                            x6
 *                       Equal and        Less than    Equal and     Less than
 *                      Over 140MHz        140MHz     Over 140MHz     140MHz
 *                          0y01            0y10          0y01         0y10
*/
	// for 196 Mhz: # ( (0x6 << 4) | (0x1 << 2) | (0x1 << 0) )
	mov r1, # ( (0x6 << 4) | (0x1 << 2) | (0x1 << 0) )
	str r1, [r0, #SYSCR4]

	# Syscr5, set every timer to the slow
	# clock and the DMC to max (no div)

/*
 * [31:7]               Undefined Read undefined. Write as zero.
 * [6]                  Undefined Read undefined. Write as one.
 * [5:4]                Undefined Read undefined. Write as zero.
 * [3]              R/W 0y1       SMC_MCLK selection register
 *        SEL_SMC_
 *                                1: fHCLK
 *       MCLK
 *                                0: fHCLK /2
 * [2]              R/W 0y1       Selection of a prescaler clock for Timer45
 *        SEL_TIM45
 *                                   0: fs(32.768KHz) clock
 *                                   1: fHCLK/2
 * [1]              R/W 0y1       Selection of a prescaler for Timer23
 *        SEL_TIM23
 *                                   0 : fs(32.768KHz) clock
 *                                   1 : fHCLK /2
 * [0]              R/W 0y1       Selection of a prescaler for Timer01
 *        SEL_TIM01
 *                                   0: fs(32.768KHz) clock
 *                                   1: fHCLK/2
*/
	mov r1, #( (0 <<3) | (1<<6) )
	str r1, [r0, #CLKCR5]

	mov pc,lr

