/*
 * Copyright (C) 2008 bplan GmbH. All rights reserved.
 *
 * universal asm mini position independant console for TMPA910 dev board
 * ARM/asm code
 *
*/
				
	.align

	.global minicon_init
	.global minicon_end

	.global minicon_putc
	.global minicon_getc

	.global minicon_getctrlbase
	.global minicon_getctrlsize

/*********/
/*********/
.set UART0DR,    0x000
.set UART0RSR,   0x004
.set UART0ECR,   0x004
.set UART0FR,    0x018
.set UART0ILPR,  0x020
.set UART0IBRD,  0x024
.set UART0FBRD,  0x028
.set UART0LCR_H, 0x02C
.set UART0CR,    0x030
.set UART0IFLS,  0x034
.set UART0IMSC,  0x038
.set UART0RIS,   0x03C
.set UART0MIS,   0x040
.set UART0ICR,   0x044
.set UART0DMACR, 0x048


.macro GET_HWBASE __reg
	mov	\__reg, #0xf2000000
.endm

/*********/
/*********/
#include "minicon_common_inline.S"

/*********/
/*********/
/*
 * return the bus addr of the ctrl				
*/				
minicon_getctrlbase:
	GET_HWBASE r0
	mov pc,lr

minicon_getctrlsize:
	mov r0,#0x50
	mov pc,lr

/*********/
/*********/
/*
 * Init the minicon facilty
 * no arg
 * r0, r1: scratch				
*/				
minicon_init:
	.type	minicon_init,#function

	/* UART1 base + 0x84 -> ctrl reg
	* add the tx enable bit
	*/

	// test if r0 is -1
	add r1,r0,#1
	cmp r1,#0
	bne _minicon_init_basevalid

	GET_HWBASE r0

/*********/
/*********/
_minicon_init_basevalid:
	SET_CTRLBASE r0

	# set the speed to 115000 bauds for 96Mhz UART Clock
	# integer = 0x34, fraction = 0x05
	mov r1,#0x34
	str r1, [r0, #UART0IBRD]

	mov r1,#0x05
	str r1, [r0, #UART0FBRD]

	# 8bit, 1 bit stop...
	# [6:5] WLEN R/W 0y00 Word length:
	#                     0y00: 5 bits, 0y01: 6 bits,
	#                     0y10: 7 bits, 0y11: 8 bits
	# [4] FEN R/W 0y0 Enable FIFOs
	#                0y1: FIFO mode
	#                0y0: Character mode
	mov r1,#(0x3<<5 | 0x1<<4)
	str r1, [r0, #UART0LCR_H]

	# Enable the controller
	# [9]   RXE     R/W  0y1  UART receive enable
	#                         0y0: Disable
	#                         0y1: Enable
	# [8]   TXE     R/W  0y1  UART transmit enable
	#                         0y0: Disable
	#                         0y1: Enable
	# [0]   UARTEN  R/W  0y0  UART enable
	#                         0y0: Disable
	#                         0y1: Enable
	mov r1, 		#( (1<<8) |  (1<<9) )
	orr r1,r1,	#(1<<0) 
	str r1, [r0, #UART0CR]

	GET_CTRLBASE r0

	mov pc,lr

/*********/
/*********/
/*
 * End			
*/				
minicon_end:
	.type	minicon_init,#function
	GET_CTRLBASE r0

	mov r1,#0
	str r1, [r0, #UART0CR]

	mov pc,lr

/*********/
/*********/

/*
 * Write a single char on the debug output
 * r0 -> the char to write
 * r1 -> scratch
*/
minicon_putc:
	.type	minicon_putc,#function

	@ poll the status reg
	GET_CTRLBASE r1

	# [3] BUSY RO 0y0 Busy flag:
	#                 0y1: The UART is transmitting data. (Busy)
	#                 0y0: The UART has stopped transmitting data.
  ldr     r1, [r1, #UART0FR]
  ands    r1, r1, #(1<<3)
	bne _minicon_error

	@ TX rd, los
	GET_CTRLBASE r1

	str	r0, [r1, #UART0DR]		@ Transmit reg
	mov	pc,lr

/*********/
/*********/
minicon_getc:
  .type   minicon_putc,#function
	

	@ poll the status reg
	GET_CTRLBASE r1

	str r1, [r1, #UART0ECR]

	# [4] RXFE RO 0y1 Receive FIFO empty flag:
	#                 0y1: Empty
	#                 0y0: Not empty
	ldr	r1, [r1, #UART0FR]
	ands r1,r1,#0x10
	bne _minicon_error

	@ RX rd, los
	GET_CTRLBASE r1
	
	ldrb	r0, [r1, #UART0DR]

  mov      pc,lr


