/*
 * Copyright (C) 2008 bplan GmbH. All rights reserved.
 *
 * Boot strap for TMPA910
 * ARM/asm code
 *
 * - Setup the clocks
 * - Init minicon to obtain an output
 * - Bring the external RAM to live
 * - (copy and) call the higher level FW
*/


#include "low_defs.i"

#define APP_RAMLOCATION	0x41000000
#define APP_NORLOCATION	0x20040000
#define APP_SIZE				0x00200000


#define APP2_RAMLOCATION	0x43000000
#define APP2_NORLOCATION	0x20600000
#define APP2_SIZE				  0x00300000
/********/
/********/
	.align

	.section ".text"

/********/
/********/

.macro PUTS __thestr__
	#adr	r0,\__thestr__
	#bl	minicon_puts
.endm

/********/
/********/

_start:
	.type	_start,#function

	@ Make sure we are in supervisor mode
	@ and not interrupts
	mov r0, #0xd3
	msr CPSR, r0

	// Put a stack at the end of the SRAM
	// the stack is going down
  ldr sp, =(SRAM_BASE+SRAM_SIZE)

	# invert the LED to show some sign of live
	mov r0,    #0xf0000000
	add r0,r0, #0x00800000
	add r0,r0, #0x00001300
	add r0,r0, #0x000000FC
	ldr	r1, [r0, #0]
	mvn r1,r1
	str	r1, [r0, #0]
				
	// active periph reset
	#bl _resetout_active

	# first setup every clocks
	bl _init_clocks

	# setup the IO as wiched for this board
	bl _init_ios

	@ Says hallo
	@ At this point, it is already good if we got
	@ something on the other end
	@ debug_init MUST preserve r7
	mov r0,#0
	add r0,r0,#-1

	bl	minicon_init

	PUTS __str_MemoryInit
	bl _init_mem
	PUTS __str_Passed

	# Init various devices
	PUTS __str_DevicesInit

	# the Static memory controller
	bl _init_smc0

	bl _init_ts
	bl _init_lcd
	bl _init_i2s
	bl _init_spi

	# allow ethernet chipset access through
	# SMC CS1
	bl _init_eth_dm9000


	bl _init_misc

	bl _init_sdhc


	PUTS __str_Passed

	PUTS __str_CopyFW
	// copy from to 

	mov r0,#APP_RAMLOCATION
	ldr r1,=APP_NORLOCATION
	mov r2,#APP_SIZE
	bl _memcpy_arm

#ifdef APP2_RAMLOCATION
	mov r0,#APP2_RAMLOCATION
	ldr r1,=APP2_NORLOCATION
	mov r2,#APP2_SIZE
	bl _memcpy_arm
#endif

	PUTS __str_Passed

	PUTS __str_CallFW


	mov r0,#0
  ldr r1,=1728
  mov r2,#0
	mov pc,#APP_RAMLOCATION

	PUTS __str_NoBoot
_noendloop:
	b	_noendloop

/********/
/********/

/*
 * some string
*/

__str_VeryLowLevelInit:	
	.asciz "Very low level init...       "

__str_MemoryInit:	
	.asciz "Memory Init...               "

__str_DevicesInit:
	.asciz "Devices Init...              "

__str_CallFW:	
	.asciz "Pass the control to Firmware\n\r"

__str_CopyFW:	
	.asciz "Copy the Firmware to memory  "

__str_Passed:	
	.asciz "Done\n\r\0"

__str_NoBoot:	
	.asciz "!!!! Boot loader failed !!!!\n\r"

.align

/********/
/********/



#include "memcpy.S"


